@@ -321,15 +321,8 @@ get_one_option(const struct my_option *opt, const char *argument,
   case 'S':
     if (filename[0] == '\0')
     {
-      /*
-        Socket given on command line, switch protocol to use SOCKETSt
-        Except on Windows if 'protocol= pipe' has been provided in
-        the config file or command line.
-      */
-      if (opt_protocol != MYSQL_PROTOCOL_PIPE)
-      {
-        opt_protocol= MYSQL_PROTOCOL_SOCKET;
-      }
+      /* Socket given on command line, switch protocol to use SOCKETSt */
+      opt_protocol= MYSQL_PROTOCOL_SOCKET;
     }
     break;
   }
@@ -369,7 +362,7 @@ int main(int argc,char *argv[])
   }
   commands = temp_argv;
   if (tty_password)
-    opt_password = my_get_tty_password(NullS);
+    opt_password = get_tty_password(NullS);
 
   (void) signal(SIGINT,endprog);			/* Here if abort */
   (void) signal(SIGTERM,endprog);		/* Here if abort */
@@ -400,6 +393,7 @@ int main(int argc,char *argv[])
     mysql_options(&mysql,MYSQL_OPT_PROTOCOL,(char*)&opt_protocol);
   if (!strcmp(default_charset,MYSQL_AUTODETECT_CHARSET_NAME))
     default_charset= (char *)my_default_csname();
+  my_set_console_cp(default_charset);
   mysql_options(&mysql, MYSQL_SET_CHARSET_NAME, default_charset);
   error_flags= (myf)(opt_nobeep ? 0 : ME_BELL);
 
@@ -1099,8 +1093,8 @@ static int execute_commands(MYSQL *mysql,int argc, char **argv)
       else if (argc == 1)
       {
         /* prompt for password */
-        typed_password= my_get_tty_password("New password: ");
-        verified= my_get_tty_password("Confirm new password: ");
+        typed_password= get_tty_password("New password: ");
+        verified= get_tty_password("Confirm new password: ");
         if (strcmp(typed_password, verified) != 0)
         {
           my_printf_error(0,"Passwords don't match",MYF(ME_BELL));
