@@ -137,6 +137,7 @@ int main(int argc, char **argv)
 
   if (!strcmp(default_charset,MYSQL_AUTODETECT_CHARSET_NAME))
     default_charset= (char *)my_default_csname();
+  my_set_console_cp(default_charset);
   mysql_options(&mysql, MYSQL_SET_CHARSET_NAME, default_charset);
 
   if (opt_plugin_dir && *opt_plugin_dir)
@@ -345,15 +346,8 @@ get_one_option(const struct my_option *opt, const char *argument,
   case 'S':
     if (filename[0] == '\0')
     {
-      /*
-        Socket given on command line, switch protocol to use SOCKETSt
-        Except on Windows if 'protocol= pipe' has been provided in
-        the config file or command line.
-      */
-      if (opt_protocol != MYSQL_PROTOCOL_PIPE)
-      {
-        opt_protocol= MYSQL_PROTOCOL_SOCKET;
-      }
+      /* Socket given on command line, switch protocol to use SOCKETSt */
+      opt_protocol= MYSQL_PROTOCOL_SOCKET;
     }
     break;
     break;
@@ -384,7 +378,7 @@ get_options(int *argc,char ***argv)
     exit(ho_error);
   
   if (tty_password)
-    opt_password=my_get_tty_password(NullS);
+    opt_password=get_tty_password(NullS);
   if (opt_count)
   {
     /*
