@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1301 USA */
 
 #ifndef WSREP_TRANS_OBSERVER_H
 #define WSREP_TRANS_OBSERVER_H
@@ -220,18 +220,13 @@ static inline bool wsrep_run_commit_hook(THD* thd, bool all)
 
   mysql_mutex_lock(&thd->LOCK_thd_data);
   /* Transaction creating sequence is TOI or RSU,
-  CREATE SEQUENCE = CREATE + INSERT (initial value)
+  CREATE [TEMPORARY] SEQUENCE = CREATE + INSERT (initial value)
   and replicated using statement based replication, thus
-  the commit hooks will be skipped.
-
-  For TEMPORARY SEQUENCES commit hooks will be done as
-  CREATE + INSERT is not replicated and needs to be
-  committed locally. */
+  the commit hooks will be skipped */
   if (ret &&
       (thd->wsrep_cs().mode() == wsrep::client_state::m_toi ||
        thd->wsrep_cs().mode() == wsrep::client_state::m_rsu) &&
-      thd->lex->sql_command == SQLCOM_CREATE_SEQUENCE &&
-      !thd->lex->tmp_table())
+      thd->lex->sql_command == SQLCOM_CREATE_SEQUENCE)
     ret= false;
   mysql_mutex_unlock(&thd->LOCK_thd_data);
 
@@ -261,11 +256,6 @@ static inline int wsrep_before_prepare(THD* thd, bool all)
                    thd->wsrep_trx().ws_meta().gtid(),
                    wsrep_gtid_server.gtid());
   }
-
-  mysql_mutex_lock(&thd->LOCK_thd_kill);
-  if (thd->killed) wsrep_backup_kill_for_commit(thd);
-  mysql_mutex_unlock(&thd->LOCK_thd_kill);
-
   DBUG_RETURN(ret);
 }
 
@@ -299,12 +289,14 @@ static inline int wsrep_before_commit(THD* thd, bool all)
   WSREP_DEBUG("wsrep_before_commit: %d, %lld",
               wsrep_is_real(thd, all),
               (long long)wsrep_thd_trx_seqno(thd));
+  THD_STAGE_INFO(thd, stage_waiting_certification);
   int ret= 0;
   DBUG_ASSERT(wsrep_run_commit_hook(thd, all));
+
   if ((ret= thd->wsrep_cs().before_commit()) == 0)
   {
     DBUG_ASSERT(!thd->wsrep_trx().ws_meta().gtid().is_undefined());
-    if (!thd->variables.gtid_seq_no && 
+    if (!thd->variables.gtid_seq_no &&
         (thd->wsrep_trx().ws_meta().flags() & wsrep::provider::flag::commit))
     {
         uint64 seqno= 0;
@@ -333,11 +325,6 @@ static inline int wsrep_before_commit(THD* thd, bool all)
                    wsrep_gtid_server.gtid());
     wsrep_register_for_group_commit(thd);
   }
-
-  mysql_mutex_lock(&thd->LOCK_thd_kill);
-  if (thd->killed) wsrep_backup_kill_for_commit(thd);
-  mysql_mutex_unlock(&thd->LOCK_thd_kill);
-
   DBUG_RETURN(ret);
 }
 
@@ -356,8 +343,7 @@ static inline int wsrep_before_commit(THD* thd, bool all)
 static inline int wsrep_ordered_commit(THD* thd, bool all)
 {
   DBUG_ENTER("wsrep_ordered_commit");
-  WSREP_DEBUG("wsrep_ordered_commit: %d %lld", wsrep_is_real(thd, all),
-              (long long) wsrep_thd_trx_seqno(thd));
+  WSREP_DEBUG("wsrep_ordered_commit: %d", wsrep_is_real(thd, all));
   DBUG_ASSERT(wsrep_run_commit_hook(thd, all));
   DBUG_RETURN(thd->wsrep_cs().ordered_commit());
 }
@@ -465,18 +451,10 @@ int wsrep_after_statement(THD* thd)
               wsrep::to_c_string(thd->wsrep_cs().state()),
               wsrep::to_c_string(thd->wsrep_cs().mode()),
               wsrep::to_c_string(thd->wsrep_cs().transaction().state()));
-  int ret= ((thd->wsrep_cs().state() != wsrep::client_state::s_none &&
+  DBUG_RETURN((thd->wsrep_cs().state() != wsrep::client_state::s_none &&
                thd->wsrep_cs().mode() == Wsrep_client_state::m_local) &&
               !thd->internal_transaction() ?
               thd->wsrep_cs().after_statement() : 0);
-
-  if (wsrep_is_active(thd))
-  {
-    mysql_mutex_lock(&thd->LOCK_thd_kill);
-    wsrep_restore_kill_after_commit(thd);
-    mysql_mutex_unlock(&thd->LOCK_thd_kill);
-  }
-  DBUG_RETURN(ret);
 }
 
 static inline void wsrep_after_apply(THD* thd)
