@@ -237,8 +237,6 @@ fail:
 
   /* Remove doublewrite pages from LRU */
   buf_pool_invalidate();
-
-  ib::info() << "Doublewrite buffer created";
   goto start_again;
 }
 
@@ -350,7 +348,7 @@ func_exit:
 /** Process and remove the double write buffer pages for all tablespaces. */
 void buf_dblwr_t::recover()
 {
-  ut_ad(recv_sys.parse_start_lsn);
+  ut_ad(log_sys.last_checkpoint_lsn);
   if (!is_created())
     return;
 
@@ -368,13 +366,13 @@ void buf_dblwr_t::recover()
       continue;
 
     const lsn_t lsn= mach_read_from_8(page + FIL_PAGE_LSN);
-    if (recv_sys.parse_start_lsn > lsn)
+    if (log_sys.last_checkpoint_lsn > lsn)
       /* Pages written before the checkpoint are not useful for recovery. */
       continue;
-    const ulint space_id= page_get_space_id(page);
+    const uint32_t space_id= page_get_space_id(page);
     const page_id_t page_id(space_id, page_no);
 
-    if (recv_sys.scanned_lsn < lsn)
+    if (recv_sys.lsn < lsn)
     {
       ib::info() << "Ignoring a doublewrite copy of page " << page_id
                  << " with future log sequence number " << lsn;
