@@ -107,15 +107,6 @@ struct pfs_os_file_t
 #endif
 };
 
-/** The next value should be smaller or equal to the smallest sector size used
-on any disk. A log block is required to be a portion of disk which is written
-so that if the start and the end of a block get written to disk, then the
-whole block gets written. This should be true even in most cases of a crash:
-if this fails for a log block, then it is equivalent to a media failure in the
-log. */
-
-#define OS_FILE_LOG_BLOCK_SIZE		512U
-
 /** Options for os_file_create_func @{ */
 enum os_file_create_t {
 	OS_FILE_OPEN = 51,		/*!< to open an existing file (if
@@ -221,10 +212,6 @@ public:
   bool is_LRU() const { return (type & (WRITE_LRU ^ WRITE_ASYNC)) != 0; }
   bool is_async() const { return (type & (READ_SYNC ^ READ_ASYNC)) != 0; }
 
-  void write_complete(int io_error) const;
-  void read_complete(int io_error) const;
-  void fake_read_complete(os_offset_t offset) const;
-
   /** If requested, free storage space associated with a section of the file.
   @param off   byte offset from the start (SEEK_SET)
   @param len   size of the hole in bytes
@@ -468,7 +455,6 @@ bool os_file_close_func(os_file_t file);
 
 /* Keys to register InnoDB I/O with performance schema */
 extern mysql_pfs_key_t	innodb_data_file_key;
-extern mysql_pfs_key_t	innodb_log_file_key;
 extern mysql_pfs_key_t	innodb_temp_file_key;
 
 /* Following four macros are instumentations to register
@@ -1054,11 +1040,6 @@ int os_aio_init();
 Frees the asynchronous io system. */
 void os_aio_free();
 
-/** Submit a fake read request during crash recovery.
-@param type   fake read request
-@param offset additional context */
-void os_fake_read(const IORequest &type, os_offset_t offset);
-
 /** Request a read or write.
 @param type		I/O request
 @param buf		buffer
